package com.topcoder.persistence.relation;

import java.util.Iterator;

import com.topcoder.persistence.relation.constraints.MathExpressionConstraint;
import com.topcoder.persistence.relation.operations.ConditionalJoinOperation;
import com.topcoder.persistence.relation.operations.DifferenceOperation;
import com.topcoder.persistence.relation.operations.EqualsJoinOperation;
import com.topcoder.persistence.relation.operations.IntersectionOperation;
import com.topcoder.persistence.relation.operations.NaturalJoinOperation;
import com.topcoder.persistence.relation.operations.ProductOperation;
import com.topcoder.persistence.relation.operations.ProjectOperation;
import com.topcoder.persistence.relation.operations.RestrictOperation;
import com.topcoder.persistence.relation.operations.UnionOperation;
import com.topcoder.persistence.relation.persistence.ResultSetPersistence;
import com.topcoder.persistence.relation.persistence.XmlPersistence;

public class Demo {
	
	private void a() {
		
		// get the manager instance
		RelationManager manager = new RelationManager();
		RelationManager manager = new RelationManager("custom.namespaces");
		
		// get the defined relation names
		String[] relationNames = manager.getRelationNames();
		
		// get a relation by name
		Relation relation = manager.getRelation(relationNames[0]);
		
		// remove a relation by name
		manager.removeRelation(relationNames[0]);
		
		// add a relation
		manager.addRelation(relation);
		
		// clear relations
		manager.clearRelations();
		
		
		
		
		// create a tuple type and a new relation
		TupleType tupleType = new TupleType(new String[] {"weight", "age"}, new Class[] {Integer.class, Integer.class});
		Relation relation = new DefaultRelation("rel1", tupleType);
		
		// add tuple constraint
		relation.addTupleConstraint("tc1", tupleConstraint);
		
		// remove tuple constraint
		relation.removeTupleConstraint("tc1");
		
		// clear tuple constraints
		relation.clearTupleConstraints();
		
		// get constraints names
		String[] constraintNames = relation.getTupleConstraintNames();
		
		// get constraint by name
		TupleConstraint constraint = relation.getTupleConstraint(constraintNames[0]);
		
		// copy constraints from another relation
		relation.copyTupleConstraints(anotherRelation);
		
		// add relation constraint
		relation.addRelationConstraint("rc1", relationConstraint);

		// remove relation constraint
		relation.removeTupleConstraint("rc1");

		// clear relation constraint
		relation.clearTupleConstraints();
		
		// get constraints names
		String[] constraintNames = relation.getRelationConstraintNames();
		
		// get constraint by name
		RelationConstraint constraint = relation.getRelationConstraint(constraintNames[0]);
		
		// copy constraints from another relation
		relation.copyRelationConstraints(anotherRelation);

		// disable relation constraints checking
		relation.setCheckRelationConstraints(false);
		
		// are relation constraints checked?
		System.out.println(relation.isCheckRelationConstraints());
		
		// create a new tuple
		Tuple tuple = new Tuple(new String[] {"weight", "age"}, new Object[] {new Integer(30), new Integer(10)});
		
		// add the tuple
		relation.addTuple(tuple);
		
		// remove the tuple
		relation.removeTuple(tuple);
		
		// clear all tuples
		relation.clearTuples();
		
		// get all tuples as array
		Tuple[] tuples = relation.getAllTuples();
		
		// get an interator to the tuples
		Iterator it = relation.getTuplesIterator();
		
		// find tuples by weight
		Tuple[] tuples = relation.findTuples("weight", new Integer(30));
		
		// find tuples by wreight and age
		Tuple[] tuples = relation.findTuples(new String[] {"weight", "age"}, new Object[] {new Integer(30), new Integer(10)});
		
		// create a result
		Relation result = new DefaultRelation("result");
		
		// 
		new IntersectionOperation().compute(relation1, relation2, result);
		new UnionOperation().compute(relation1, relation2, result);
		new DifferenceOperation().compute(relation1, relation2, result);
		new ProjectOperation(new String[] {"weight"}).compute(sourceRelation, result);
		new RestrictOperation(new MathExpressionConstraint("weight * 2 < age")).compute(sourceRelation, result);
		new NaturalJoinOperation().compute(relation1, relation2, result);
		new ProductOperation().compute(relation1, relation2, result);
		new EqualsJoinOperation(new String[] {"a", "b"}, new String[] {"a", "x"}).compute(relation1, relation2, result);
		new ConditionalJoinOperation(new MathExpressionConstraint("left.a > right.x")).compute(relation1, relation2, result);
		
		Relation loaded = new DefaultRelation("rel2");
		new XmlPersistence("file.xml").load(loaded);
		new XmlPersistence("file.xml").save(relation);
		
		Relation loaded = new DefaultRelation("rel3");
		new ResultSetPersistence(resultSet).load(loaded);
	}

}
