package com.topcoder.persistence.relation;

/**
 * The PersistenceException exception is used to wrap any persistence implementation specific exception (such as I/O
 * exception, SQL exception or XML exception).
 *
 * These exceptions are thrown by the persistence interface implementations. Since they are implementation
 * specific, there needs to be a common way to report them to the user, and that's what this exception is used for.
 *
 * Thread safety: 
 * 
 * The class is immutable so it is thread safe.
 *
 * @author TCSDESIGNER
 */
public class PersistenceException extends RelationException {

    /**
     * Create a new instance ConfigurationException with the given error message.
     *
     * The message is enforced to be completed, so that the user will benefit from meaningful messages.
     *
     * Simply calls the super constructor with the message.
     *
     * @param message the message describing the exception
     *
     * @throws IllegalArgumentException if the message is null
     */
    public PersistenceException(String message) throws IllegalArgumentException {
        super(message);
    }

    /**
     * Create a new instance ConfigurationException with the given error message and the given cause.
     *
     * The message is enforced to be completed, so that the user will benefit from meaningful messages.
     *
     * Simply calls the super constructor with the message and cause.
     *
     * @param message the message describing the exception
     * @param cause the cause of the exception
     *
     * @throws IllegalArgumentException if the message is null or if cause is null
     */
    public PersistenceException(String message, Throwable cause) throws IllegalArgumentException {
        super(message, cause);
    }

}
