package com.topcoder.persistence.relation;


/**
 * This interface abstracts a constraint applied on a tuple.
 * 
 * The interface defines only one method that takes as input a tuple and
 * returns a boolean value indicating whether the constraint is met or not.
 * 
 * Thread safety:
 * 
 * The implementations of this class are expected to have no state or be
 * immutable so they should be thread safe.
 * 
 * @author TCSDESIGNER
 */
public interface TupleConstraint {
	
	/**
	 * Checks whether the tuple respects the constraint.
	 * 
	 * @param tuple the tuple to check
	 * @return true if constraint is met, false otherwise
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException thrown if the evaluation cannot be done because of an
	 * implementation specific error 
	 */
	public boolean validate(Tuple tuple) throws IllegalArgumentException, EvaluationException;

}
