package com.topcoder.persistence.relation;

import java.util.HashMap;
import java.util.Map;

/**
 * This class represents a tuple type.
 * A tuple type is a set of attribute names and their types.
 * 
 * In addition to storing the attribute names and their types,
 * this class has also some useful methods such as testing whether
 * a Tuple matches the tuple type (is an instance of the tuple type,
 * meaning the tuple values match the attribute types).
 * 
 * It also implements equals to ease the testing of whether two
 * tuple types are the same. 
 * 
 * Thread safety:
 * 
 * This class is immutable so it is thread safe.
 * 
 * @author TCSDESIGNER
 */
public class TupleType {
	
	/**
	 * The attribute name to attribute type mapping.
	 * Maps strings to Classes.
	 * Cannot be null or have null keys and values.
	 * Initialized with HashMap.
	 * Filled in the constructor and used in all the other methods.
	 */
	private Map types = new HashMap();
	
	/**
	 * Constructor.
	 * 
	 * Implementation details:
	 * 
	 * Puts all attribute name - attribute type pairs in the types map.
	 *  
	 * @param attributes the attribute names
	 * @param types the attribute types
	 * 
	 * @throws IllegalArgumentException if the arguments are null, have null element,
	 * have different sizes or if the Strings are empty or all spaces
	 */
	public TupleType(String[] attributes, Class[] types) throws IllegalArgumentException {
	}
	
	/**
	 * Returns the attribute names.
	 * 
	 * Implementation details:
	 * 
	 * Build the array from the key set of the types map.
	 * 
	 * @return the attribute names
	 */
	public String[] getAttributes() {
		return null;
	}
	
	/**
	 * Returns the attribute type of an attribute.
	 * 
	 * Implementation details:
	 * 
	 * Gets the type from the map.
	 * 
	 * @param attribute the attribute name
	 * 
	 * @return the type of the attribute (null if no such attribute exists)
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public Class getType(String attribute) throws IllegalArgumentException {
		return null;
	}
	
	/**
	 * Helper method that determines whether a tuple is an instance of the tuple type.
	 * 
	 * Implementation details:
	 * 
	 * - checks if the sets of attribute names (from the tuple type and tuple) are equal
	 * - each tuple value must be instance of corresponding tuple type (Class.isInstance could be used). 
	 * 
	 * @param tuple the tuple
	 * 
	 * @return true if an instance, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public boolean isInstance(Tuple tuple) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Returns whether this tuple type is equal with the given object
	 * 
	 * Implemenation details:
	 * 
	 * - object must be a TupleType to return true
	 * - the types map of the two objects must be equal to return true
	 * 
	 * @param object the object to compare to
	 * 
	 * @return true is equal, false is not
	 */
	public boolean equals(Object object) {
		return false;
	}
	
	/**
	 * Returns the hashcode of the object.
	 * 
	 * Implementation details:
	 * 
	 * Returns types.hashCode().
	 * 
	 * @return the hashcode
	 */
	public int hashCode() {
		return 0;
	}

}
