package com.topcoder.persistence.relation;

/**
 * Interface that abstracts any operation that takes one relation as input
 * and fills with tuples a result relation.
 *
 * The result relation is given by the user (must be empty and typeless).
 * This allows the user to select the desired relation implementation
 * (optimized for speed, for memory, etc).
 * 
 * @author TCSDESIGNER
 */
public interface UnaryOperation {

	/**
	 * Computes the result relation from the source relation.
	 * 
	 * @param sourceRelation the source relation
	 * @param resultRelation the result relation (must be empty and typeless)
	 * 
	 * @throws IllegalArgumentException if any argument is null or the result relation is not empty and typeless
	 * @throws OperationException if any implementation specific error occurs
	 */
	public void compute(Relation sourceRelation, Relation resultRelation) throws IllegalArgumentException, OperationException;

}
