package com.topcoder.persistence.relation.constraints;

import com.topcoder.persistence.relation.EvaluationException;
import com.topcoder.persistence.relation.Tuple;
import com.topcoder.persistence.relation.TupleConstraint;

/**
 * This class represents a tuple constraint that specifies that a
 * relationship must exist between two attributes of the tuples
 * from a relation.
 * 
 * For example it allows saying that workExperience <= age.
 * The comparisons can be EQUALS, LOWER, GREATER, LOWER OR EQUAL,
 * GREATER OR EQUAL.
 * 
 * This constraint uses Comparable.compareTo for comparison.
 * That means that the types used for the compared attributes must
 * implement Comparable.
 * 
 * Thread safety:
 * 
 * Immutable class so there are no thread safety issues. 
 * 
 * @author TCSDESIGNER
 */
public class AttributeComparisonConstraint implements TupleConstraint {

	/**
	 * The left attribute the check is performed on.
	 * Cannot be null, empty or all spaces.
	 * Set in the constructor, used in validate.
	 */
	private String leftAttribute = null;

	/**
	 * The right attribute the check is performed on.
	 * Cannot be null, empty or all spaces.
	 * Set in the constructor, used in validate.
	 */
	private String rightAttribute = null;
	
	/**
	 * The comparison type.
	 * Can be null (meaning EQUALS should be used).
	 * Set in the constructor, used in validate.
	 */
	private ComparisonType comparisonType = null;
	
	/**
	 * Constructor. Copies the params to the attributes. 
	 * 
	 * @param leftAttribute the left attribute to compare
	 * @param rightAttribute the right attribute to compare
	 * 
	 * @throws IllegalArgumentException if any argument is null or empty or all spaces
	 */
	public AttributeComparisonConstraint(String leftAttribute, String rightAttribute) throws IllegalArgumentException {
	}

	/**
	 * Constructor. Copies the params to the attributes. 
	 * 
	 * @param leftAttribute the left attribute to compare
	 * @param rightAttribute the right attribute to compare
	 * @param comparisonType the comparison type
	 * 
	 * @throws IllegalArgumentException if any argument is null or if any attribute is empty or all spaces
	 */
	public AttributeComparisonConstraint(String leftAttribute, String rightAttribute, ComparisonType comparisonType) throws IllegalArgumentException {
	}

	/**
	 * Getter for the left attribute (leftAttribute field).
	 * 
	 * @return the left attribute
	 */
	public String getLeftAttribute() {
		return null;
	}

	/**
	 * Getter for the right attribute (rightAttribute field).
	 * 
	 * @return the right attribute
	 */
	public String getRightAttribute() {
		return null;
	}

	/**
	 * Getter for the comparison type (comparisonType field).
	 * 
	 * @return the comparison type
	 */
	public ComparisonType getComparisonType() {
		return null;
	}

	/**
	 * Checks whether the tuple respects the constraint.
	 * 
	 * Implementation details:
	 * 
	 * - leftAttrValue = tuple.getValue(leftAttribute)
	 * - rightAttrValue = tuple.getValue(rightAttribute)
	 * - EQUAL or no type: return leftAttrValue.equals(rightAttrValue) 
	 * - LESS: return leftAttrValue.compareTo(rightAttrValue) < 0 
	 * - GREATER: return leftAttrValue.compareTo(rightAttrValue) > 0 
	 * - LESS OR EQUAL: return leftAttrValue.compareTo(rightAttrValue) <= 0 
	 * - GREATER OR EQUAL: return leftAttrValue.compareTo(rightAttrValue) >= 0 
	 * 
	 * @param tuple the tuple to check
	 * 
	 * @return true if constraint is met, false otherwise
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException thrown if the evaluation cannot be done because of an
	 * implementation specific error 
	 */
	public boolean validate(Tuple tuple) throws IllegalArgumentException, EvaluationException {
		return false;
	}
	
}
