package com.topcoder.persistence.relation.constraints;

import com.topcoder.persistence.relation.EvaluationException;
import com.topcoder.persistence.relation.Tuple;
import com.topcoder.persistence.relation.TupleConstraint;

/**
 * This class represents a tuple constraint that specifies that a
 * relationship must exist between an attributes and a given value
 * 
 * For example it allows saying that age <= 20 or that name >= "B".
 * The comparisons can be EQUALS, LOWER, GREATER, LOWER OR EQUAL,
 * GREATER OR EQUAL.
 * 
 * This constraint uses Comparable.compareTo for comparison.
 * That means that the types used for the compared attribute must
 * implement Comparable.
 * 
 * Thread safety:
 * 
 * Immutable class so there are no thread safety issues. 
 * 
 * @author TCSDESIGNER
 */
public class ComparisonConstraint implements TupleConstraint {
	
	/**
	 * The attribute the check is performed on.
	 * Cannot be null, empty or all spaces.
	 * Set in the constructor, used in validate.
	 */
	private String attribute = null;
	
	/**
	 * The value the attribute is compared to.
	 * Cannot be null.
	 * Set in the constructor, used in validate.
	 */
	private Object value = null;
	
	/**
	 * The comparison type.
	 * Can be null (meaning EQUALS should be used).
	 * Set in the constructor, used in validate.
	 */
	private ComparisonType comparisonType = null;
	
	/**
	 * Constructor. Copies the params to the attributes. 
	 * 
	 * @param attribute the attribute to compare
	 * @param value the value to compare the attribute to
	 * @throws IllegalArgumentException if any argument is null or if attribute is empty or all space
	 */
	public ComparisonConstraint(String attribute, Object value) throws IllegalArgumentException {
	}

	/**
	 * Constructor. Copies the params to the attributes. 
	 * 
	 * @param attribute the attribute to compare
	 * @param value the value to compare the attribute to
	 * @param comparisonType the comparison type
	 * 
	 * @throws IllegalArgumentException if any argument is null or if attribute is empty or all space
	 */
	public ComparisonConstraint(String attribute, Object value, ComparisonType comparisonType) throws IllegalArgumentException {
	}

	/**
	 * Getter for the attribute (attribute field).
	 * 
	 * @return the attribute
	 */
	public String getAttribute() {
		return null;
	}

	/**
	 * Getter for the compare value (value field).
	 * 
	 * @return the compare value
	 */
	public Object getValue() {
		return null;
	}

	/**
	 * Getter for the comparison type (comparisonType field).
	 * 
	 * @return the comparison type
	 */
	public ComparisonType getComparisonType() {
		return null;
	}

	/**
	 * Checks whether the tuple respects the constraint.
	 * 
	 * Implementation details:
	 * 
	 * - attrValue = tuple.getValue(attribute)
	 * - EQUAL or no type: return attrValue.equals(value) 
	 * - LESS: return attrValue.compareTo(value) < 0 
	 * - GREATER: return attrValue.compareTo(value) > 0 
	 * - LESS OR EQUAL: return attrValue.compareTo(value) <= 0 
	 * - GREATER OR EQUAL: return attrValue.compareTo(value) >= 0 
	 * 
	 * @param tuple the tuple to check
	 * 
	 * @return true if constraint is met, false otherwise
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException thrown if the evaluation cannot be done because of an
	 * implementation specific error 
	 */
	public boolean validate(Tuple tuple) throws IllegalArgumentException, EvaluationException {
		return false;
	}

}
