package com.topcoder.persistence.relation.constraints;

import com.topcoder.persistence.relation.EvaluationException;
import com.topcoder.persistence.relation.Tuple;
import com.topcoder.persistence.relation.TupleConstraint;

/**
 * This class represents a special type of constraint that simply negates
 * the result of another constraint. It contains the other constraint and
 * its evaluation consists of calling the contained constraint and
 * negating the result. 
 * 
 * Thread safety:
 * 
 * Immutable class so there are no thread safety issues. 
 * 
 * @author TCSDESIGNER
 */
public class NotConstraint implements TupleConstraint {
	
	/**
	 * The contained constraint.
	 * Cannot be null.
	 * Set in the constructor, used in the methods.
	 */	
	public TupleConstraint constraint = null;
	
	/**
	 * Constructor. Copies param to attribute.
	 * 
	 * @param constraint the contained constraint
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public NotConstraint(TupleConstraint constraint) throws IllegalArgumentException{
	}
	
	/**
	 * Returns the constraint (the constraint attribute).
	 * 
	 * @return the constraint
	 */
	public TupleConstraint getConstraint() {
		return null;
	}

	/**
	 * Checks whether the tuple respects the constraint.
	 * 
	 * Implementation details:
	 * 
	 * - return not constraint.validate(tuple)
	 * 
	 * @param tuple the tuple to check
	 * 
	 * @return true if constraint is met, false otherwise
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException thrown if the evaluation cannot be done because of an
	 * implementation specific error 
	 */
	public boolean validate(Tuple tuple) throws IllegalArgumentException, EvaluationException {
		return false;
	}

}
