package com.topcoder.persistence.relation.constraints;

import com.topcoder.persistence.relation.EvaluationException;
import com.topcoder.persistence.relation.Tuple;
import com.topcoder.persistence.relation.TupleConstraint;

/**
 * This class represents a tuple constraint that matches a string attribute
 * to a regular expression. This means that the constraint is considered to
 * be met if all tuples from a relation have the specified attribute values
 * matching a given regular expression.
 * 
 * Thread safety:
 * 
 * Immutable class so there are no thread safety issues. 
 * 
 * @author TCSDESIGNER
 */
public class RegularExpressionConstraint implements TupleConstraint {
	
	/**
	 * The attribute the match is performed on.
	 * Cannot be null, empty or all spaces.
	 * Set in the constructor, used in validate.
	 */
	private String attribute = null;
	
	/**
	 * The regular expression that is used to match the attribute value.
	 * Cannot be null.
	 * Set in the constructor, used in validate.
	 */    
	private String expression = null;
	
	/**
	 * Constructor. Copies the params to the attributes.
	 * 
	 * @param attribute the attribute to test
	 * @param expression the regular expression to match 
	 * 
	 * @throws IllegalArgumentException if attribute argument is null, empty string or all spaces
	 * or expression is null
	 */
	public RegularExpressionConstraint(String attribute, String expression) throws IllegalArgumentException {
	}
	
	/**
	 * Getter for the match attribute (attribute field).
	 * 
	 * @return the match attribute
	 */
	public String getAttribute() {
		return null;
	}

	/**
	 * Getter for the regular expression (expression field).
	 *  
	 * @return the expression
	 */
	public String getExpression() {
		return null;
	}

	/**
	 * Checks whether the tuple respects the constraint.
	 * 
	 * Implementation details:
	 * 
	 * - tuple.getValue(attribute).matches(expression)
	 * 
	 * @param tuple the tuple to check
	 * 
	 * @return true if constraint is met, false otherwise
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException thrown if the evaluation cannot be done because of an
	 * implementation specific error 
	 */
	public boolean validate(Tuple tuple) throws IllegalArgumentException, EvaluationException {
		return false;
	}

}
