package com.topcoder.persistence.relation.operations;

/**
 * A binary operation implementation for computing the product of two relations.
 * This will create as result a relation having tuples formed by all the pairs
 * of tuples from the two relations (combined).
 * 
 * The product is a special case of join where the constraint is something that
 * evaluates to true always (missing criteria). In order to take advantage of
 * the alias handling, it is implemented as a ConditionalJoinOperation subclass.
 *
 * Aliases are used when two relations have common attribute names.
 * Joining them would result in name conflict so aliasing allows
 * renaming fields in order to avoid that.
 *
 * The result relation is given by the user (must be empty and typeless).
 * This allows the user to select the desired relation implementation
 * (optimized for speed, for memory, etc).
 * 
 * Thread safety:
 * 
 * This class is immutable so it is thread safe.
 * 
 * @author TCSDESIGNER
 */
public class ProductOperation extends ConditionalJoinOperation {

	/**
	 * Constructor that makes a product operation with no aliases.
	 */
	public ProductOperation() {
	}
	
	/**
	 * Constructor that makes a product operation with given aliases.
	 * 
	 * Implementation details:
	 * 
	 * Calls super to handle aliases.
	 * 
     * @param leftAliased the attributes from the left relation that are aliased 
     * @param leftAliases the aliases for the aliased attributes from the left table
     * @param rightAliased the attributes from the right relation that are aliased
     * @param rightAliases the aliases for the aliased attributes from the right table
     * 
     * @throws IllegalArgumentException if any argument is null, has null elements or 
	 * empty or all spaces strings or if the sizes of the pair arrays are different
	 */
	public ProductOperation(String[] leftAliased, String[] leftAliases, String[] rightAliased, String[] rightAliases) throws IllegalArgumentException {
	}

}
