package com.topcoder.persistence.relation.persistence;

import java.sql.ResultSet;

import com.topcoder.persistence.relation.PersistenceException;
import com.topcoder.persistence.relation.Relation;
import com.topcoder.persistence.relation.RelationPersistence;

/**
 * Persistence implementation that uses a ResultSet as a source for creating a relation.
 * Saving is not supported because it is not possible (ResultSet being a data extraction only class).
 * 
 * Both the tuple type and the tuple data are retrieved from the ResultSet.
 * 
 * Thread safety:
 * 
 * An instance is meant to be used just once (because the ResultSet cannot be rewinded afterwards).
 * So there can be no multiple thread support discussion here because multiple
 * threads are not supported due to the nature of ResultSet.
 * 
 * @author TCSDESIGNER
 */
public class ResultSetPersistence implements RelationPersistence {
	
	/**
	 * The ResultSet to populate the relation from.
	 * Cannot be null.
	 * Set in the constructor, used in load.
	 */
	private ResultSet resultSet = null;
	
	/**
	 * Constructor.
	 * 
	 * Implementation details:
	 * 
	 * Copies the param to the attribute.
	 * 
	 * @param resultSet the ResultSet to use
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public ResultSetPersistence(ResultSet resultSet) throws IllegalArgumentException {
	}

	/**
	 * Saves an entire relation to persistence.
	 * Not supported.
	 * 
	 * @param relation the relation to persist
	 * 
	 * @throws PersistenceException always
	 */
	public void save(Relation relation) {
	}

	/**
	 * Loads an entire relation from the ResultSet.
	 * 
	 * Implementation details:
	 * 
	 * - resultSet.getMetaData() is used to get the field info
	 * - the field info is used to create the tuple type
	 * - iterate the resultSet until the last record is reached
	 * - for each record, a tuple is created with the values retrieved using getObject(idx)
	 * - the tuple is added to the result
	 * 
	 * @param resultRelation the result relation (must be empty and typeless)
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 * @throws PersistenceException if an implementation specific error has occured
	 */
	public void load(Relation resultRelation) throws IllegalArgumentException, PersistenceException {
	}

}
