package com.topcoder.persistence.relation.persistence;

import java.io.InputStream;
import java.io.OutputStream;

import com.topcoder.persistence.relation.PersistenceException;
import com.topcoder.persistence.relation.Relation;
import com.topcoder.persistence.relation.RelationPersistence;

/**
 * Persistence implementation that uses XML for relation persistence.
 * 
 * This version supports the following types:
 * String, Boolean, Byte, Integer, Short, Long, Float, Double, BigDecimal, BigInteger, Date.
 * Future version might support more formats or include a pluggable text to value
 * parsing framework.
 * 
 * The format of the XML can be seen in the sample file.
 * 
 * Thread safety:
 * 
 * An instance is meant to be used just once (because the streams cannot be rewinded afterwards).
 * So there can be no multiple thread support discussion here because multiple
 * threads are not supported due to the nature of the streams.
 * 
 * @author TCSDESIGNER
 */
public class XmlPersistence implements RelationPersistence {
	
	/**
	 * The XML filename to use.
	 * Can be null if at least one of the stream is not.
	 * Set in the constructor, used in save and load.
	 */
	private String filename = null;
	
	/**
	 * The input stream to use.
	 * Can be null if filename is not null or if outputStream is not null.
	 * Set in the constructor, used in load.
	 */
	private InputStream inputStream = null;
	
	/**
	 * The output stream to use.
	 * Can be null if filename is not null or if inputStream is not null.
	 * Set in the constructor, used in save.
	 */
	private OutputStream outputStream = null;
	
	/**
	 * Constructor.
	 * 
	 * Implementation details:
	 * 
	 * Copies the param to the attribute.
	 * 
	 * @param filename the filename to use
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public XmlPersistence(String filename) throws IllegalArgumentException {	
	}

	/**
	 * Constructor.
	 * 
	 * Implementation details:
	 * 
	 * Copies the params to the attributes.
	 * 
	 * @param inputStream the input stream to use
	 * @param outputStream the output stream to use
	 * 
	 * @throws IllegalArgumentException if both streams are null (one is accepted to be null)
	 */
	public XmlPersistence(InputStream inputStream, OutputStream outputStream) throws IllegalArgumentException {	
	}

	/**
	 * Saves an entire relation to XML.
	 * 
	 * Implementation details:
	 * 
	 * Since the XML output is quite simple, DOM is overkill for XML writing.
	 * The text can be written manually to the output stream using the
	 * following steps:
	 * 
	 * - if output stream is null, open an output stream for the filename
	 * - begin relation element
	 * - for each attribute write an attribute element with the name and type
	 * - for each tuple begin tuple element
	 * - for each attribute write an attribute element with the name and value
	 * - end tuple element
	 * - end relation element
	 * 
	 * @param relation the relation to persist
	 * 
	 * @throws PersistenceException always
	 */
	public void save(Relation relation) {
	}

	/**
	 * Loads an entire relation from the XML.
	 * 
	 * Implementation details:
	 * 
	 * - parse the XML using DOM from either the stream or filename (whichever is not null)
	 * - each attribute element specifies the an attribute name and type
	 * - this info is used to create the tuple type
	 * - each tuple element is then iterated
	 * - each attribute subelement contains the value of one attribute
	 * - this info is used to create a tuple that is added to the result
	 * 
	 * @param resultRelation the result relation (must be empty and typeless)
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 * @throws PersistenceException if an implementation specific error has occured
	 */
	public void load(Relation resultRelation) throws IllegalArgumentException, PersistenceException {
	}

}
